;------------------------------------------------------------
;------------------------------------------------------------
;------------------------------------------------------------
;+
; NAME:pltbase
;
; PURPOSE: surcouche de contour pour tracer un champ eventuellement
; masque. brique elementaire de plt, pltz et pltt.
;
; CATEGORY:un tarce vite fait et/ou delestage de l''ecriture de plt, pltz, pltt
;
; CALLING SEQUENCE: pltbase, z2d, x, y, mask,xm, ym, [, levels, colors],
;
; INPUTS:
;        z2d:le tableau a tracer
;
;        x et y les axes. vecteurs ou tableaux de meme taille que
;        z2d. Ce sont les coordonnees de z2d
;
;        mask: le tableau qui masque z2d (avec des 0 sur les points a
;        masquer et des 1 sur les autres). si z2d n'est pas masque
;        mettre cet argument egale a 1.
;
;        xm et ym les axes du mask. vecteurs ou tableaux de meme taille que
;        mask. Ce sont les coordonnees de mask
;
;        levels et colors: (optionnels) les vecteurs qui contiennent
;        les niveaus et les couleurs necessaires au contour. S''il ne
;        sont pas donnes on prends 20 niveau entre le min et le max.
;
; KEYWORD PARAMETERS:
;
;        COLORTRICHAMP : la couleur que l''on veut utiliser pour
;        dessiner la triangulation qui est utilisee pour faire les
;        contour du champ
;
;        COLORTRIMASK : la couleur que l''on veut utiliser pour
;        dessiner la triangulation qui est utilisee pour faire les
;        contour du mask
;
;        /CONT_NOFILL: activer pour ne pas remplir les points masques
;        a la couleur c_cont mais pour les laisser en transparent! Rq
;        on trace qd meme le contour du mask de la couleur c_cote
;
;        CONT_THICK: l''epaisseur du trait pour tracer les
;        continents. par defaut c''est 1.
;
;        /DESSTRICHAMP: pour dessiner la triangulation qui est
;        utilisee pour faire les contour du champ. 
;
;        /DESSTRIMASK: pour dessiner la triangulation qui est
;        utilisee pour faire les contour du mask
;
;        /FORPLT: a activer si on veut que le trace des cote soit
;        realise par tracecote plutot que tracemask.
;
;        I_COLORS: un vecteur specifiant la couleur a utiliser pour
;        tracer les contours. C''est la meme chose que c_colors qui
;        ajit sur les contours.
;
;        MORE: chiffre a donner pour eviter les bug du style:
;  % Out of range subscript encountered: <LONG      Array[38911]>.
;  % Execution halted at:  PLTBASE           151
;             par defaut more=10 si le bug existe tjs, augmenter la
;             valeur de more! (l''explication et la justification de
;             cette methode n''ont pas encore de fondements
;             scientifiques)...
;         
;
;
;        /NOCOULEUR: pour faire juste les isolignes
;
;        /NOCONTOUR: pour faire juste les couleurs
;
;        /UNSUR2: pour tarcer une isoligne sur 2
;
;        UNLABSUR: entier n. specifant qu''on ne labelle qu''un
;        contour sur n
;
;        CONTOUR: pour etrte utilise depuis plt, pltz ou pltt. cf ces
;        routines.
;
;        _EXTRA: mot cle magique d''idl pour faire passer tous lse
;        mots cles acceptes par les routines et fonctions utilises ds
;        ce programme sans les declarer explicitement. 
;
; OUTPUTS:
;
; COMMON BLOCKS:
;        common.pro
;
; SIDE EFFECTS:
;
; RESTRICTIONS:ds le cas ou z2d, x et y sont des tableaux de meme
; taille il faut les metre sous forme de vecteur: z2d[*], x[*], y[*].
;
; EXAMPLE:
;
; MODIFICATION HISTORY: Sebastien Masson (smasson@lodyc.jussieu.fr)
;                       ????
;                       8/2/2000 check if the tri array is not equal to
;                       -1. allow contour with out using a triangulation.
;
;-
;------------------------------------------------------------
;------------------------------------------------------------
;------------------------------------------------------------
PRO pltbase, z2d, x, y, mask,xm, ym, levels, colors,UNSUR2 = unsur2, CONTOUR = contour $
             , NOCONTOUR = nocontour, NOCOULEUR = nocouleur, TRICHAMP = trichamp, TRIMSK = trimsk $
             , CARTE = carte, NAN = nan $
             , COLORTRICHAMP = colortrichamp, COLORTRIMASK = colortrimask $
             , COLORTRINAN = colortrinan $
             , TH_TRICHAMP = th_trichamp, TH_TRIMASK = th_trimask $
             , DESSTRICHAMP = desstrichamp, DESSTRIMASK = desstrimask $
             , DESSTRINAN = desstrinan $
             , I_COLORS = i_colors $
             , CONT_THICK = cont_thick,CONT_NOFILL = cont_nofill, UNLABSUR = unlabsur $
             , COINMONTEMASK = coinmontemask, COINDESCENDMASK = coindescendmask $
             , COINMONTENAN = coinmontenan, COINDESCENDNAN = coindescendnan $
             , INDICEZOOMMASK = indicezoommask, INDICEZOOMNAN = indicezoomnan $
             , MASKNAN = masknan, TRINAN = trinan, XNAN = xnan, YNAN = ynan, FORPLT = forplt $
             , MORE = more, _EXTRA = ex
@common
;
; explication concernant contour. Ce mot cle est active qd on on trace
; un contour en couleur different de celui en trait noir.
; si il est active (cas n_elements(contour) NE 0), on passe 2 fois ds pltbase:
;   1) on trace les couleurs puis on sort c''est le cas:
;         n_elements(contour) NE 0 AND n_elements(contour) NE 4)
;   2) on trace les contour en trait puis les continents c''est le cas 
;         n_elements(contour) NE 0 AND n_elements(contour) EQ 4)
;
;------------------------------------------------------------
   tempsun = systime(1)         ; pour key_performance
   if n_elements(masknan) EQ 0 then masknan = 1
;----------------------------------------------------------------------
; si les niveaux et les couleurs ne sont pas donnes
;----------------------------------------------------------------------
   if n_params() EQ 4 then BEGIN 
      label,0,min(z2d*mask),max(z2d*mask),ncontour,levels
      colors = couleur
   endif
; attention bidouille inexplicable pour que tout se passe bien avec les
; postcript ds pltz!!!
   if n_elements(contour) LE 4 AND !x.type EQ 0 THEN $
    plot, [0], [0], xstyle = 5, ystyle = 5,/noerase, title = '',  subtitle = ''
;----------------------------------------------------------------------
; si cell_fill fait partit de _extra, on le desactive si il n''est pas
; egale a 2
;----------------------------------------------------------------------
   IF chkstru(ex, 'CELL_FILL') THEN BEGIN 
      cell_fill = ex.CELL_FILL
      if ex.CELL_FILL NE 2 then ex.CELL_FILL = 0
   ENDIF ELSE cell_fill = 0     ;
; do we use the triangulation??
   usetri = n_elements(trichamp) NE 0 AND (testvar(var = trichamp))[0] NE -1
;----------------------------------------------------------------------
; I) remplissage des contours en palette de couleur 
;----------------------------------------------------------------------
   if NOT keyword_set(more) then more = 10
   if NOT keyword_set(nocouleur) then begin
      if n_elements(contour) NE 4 THEN BEGIN 
         if usetri then $
          contour,[z2d[*], fltarr(more)],[x[*], fltarr(more)],[y[*], fltarr(more)],levels=levels,c_color=colors,/noerase,/fill,/closed $
          ,TRIANGULATION=trichamp, _extra = ex $
         else $
          contour,z2d,x,y,levels=levels,c_color=colors,/noerase,/fill,/closed, _extra = ex 
      endif
   endif
   if n_elements(contour) NE 0 AND n_elements(contour) NE 4 THEN GOTO, fini 
   IF chkstru(ex, 'C_ORIENTATION') THEN ex = extractstru(ex, 'C_ORIENTATION')
   IF chkstru(ex, 'C_SPACING') THEN ex = extractstru(ex, 'C_SPACING')
;------------------------------------------------------------
; II) trace des contours en trait de couleur c_lab
;------------------------------------------------------------
   if n_elements(contour) EQ 4 OR n_elements(contour) EQ 0 THEN BEGIN
; on ne passe pas si on doit faire des contours differents...
; ds le cas on unsur2 est active on reduit levels
      if NOT keyword_set(nocontour) then begin
         IF keyword_set(unsur2) THEN levels = levels(where(zeroun(n_elements(levels) ) eq 1))
; unlabsur est active?  C_LABEL est passe via _EXTRA?
         if keyword_set(unlabsur) THEN IF chkstru(ex, 'C_LABELS') THEN $
          ex.C_LABELS = 1-((indgen(n_elements(ex.C_LABELS)) MOD unlabsur) < 1)
;
; pour ne pas filler qd cell_fill est impose...
         IF chkstru(ex, 'CELL_FILL') THEN ex.CELL_FILL = 0
         IF chkstru(ex, 'C_COLORS') THEN ex = extractstru(ex, 'C_COLORS')
;
         if usetri then $
          contour, [z2d[*], fltarr(more)], [x[*], fltarr(more)],[y[*], fltarr(more)],levels=levels, overplot = 1-keyword_set(nocouleur) $
          ,noerase = keyword_set(nocouleur), c_color=c_lab*(1-keyword_set(i_colors))+testvar(var = i_colors),TRIANGULATION=trichamp, _extra = ex $
         else $
          contour, z2d, x,y,levels=levels, overplot = 1-keyword_set(nocouleur) $
          ,noerase = keyword_set(nocouleur), c_color=c_lab*(1-keyword_set(i_colors))+testvar(var = i_colors), _extra = ex
;
      endif
;
      if keyword_set(carte) then if carte EQ 1 then mask = 1
      if n_elements(mask) NE 1 then BEGIN ; si mask=1 on saute.
;----------------------------------------------------------------------
; III) remplissage des continents de couleur c_cont
;----------------------------------------------------------------------
         if NOT keyword_set(cont_nofill) then BEGIN
            IF chkstru(ex, 'CELL_FILL') THEN ex.CELL_FILL = cell_fill < 1
            IF chkstru(ex, 'LEVELS') THEN ex = extractstru(ex, 'LEVELS')
            if n_elements(trimsk) eq 0 and usetri then trimsk = trichamp
;
            usetri = n_elements(trimsk) NE 0 AND (testvar(var = trimsk))[0] NE -1
            if usetri then BEGIN
               if keyword_set(forplt) then $
                contour, -1*[mask[*], fltarr(more)], [xm[*], fltarr(more)], [ym[*], fltarr(more)], levels=-0.5, /overplot, /fill ,c_color=c_cont $
                ,TRIANGULATION=trimsk, _extra=ex ELSE $
                contour, -1*[(mask*masknan)[*], fltarr(more)], [xm[*], fltarr(more)], [ym[*], fltarr(more)], levels=-0.5, /overplot, /fill $
                ,c_color=c_cont,TRIANGULATION=trimsk, _extra=ex ;
               completecointerre, coinmonte=coinmontemask, coindescend=coindescendmask $
                , indicezoom = indicezoommask, _extra = ex
            ENDIF ELSE BEGIN
               if keyword_set(forplt) then $
                contour, -1*mask, xm, ym, levels=-0.5, /overplot, /fill ,c_color=c_cont $
                , _extra=ex ELSE $
                contour, -1*(mask*masknan), xm, ym, levels=-0.5, /overplot, /fill $
                ,c_color=c_cont, _extra=ex ;
            ENDELSE
;

; si il y a des points a nan et que le champ n''est pas sur la grille T
; on trace en blanc les points a nan avant de dessiner les cotes avec
; un trait.
            if keyword_set(trinan) THEN BEGIN
               contour, [-masknan[*], fltarr(more)],[xnan[*], fltarr(more)],[ynan[*], fltarr(more)],levels=-0.5,/overplot,/fill,c_color=c_cont $
                , TRIANGULATION=trinan, _extra = ex
               completecointerre, coinmonte=coinmontenan, coindescend=coindescendnan $
                , indicezoom = indicezoomnan, _extra = ex
               if keyword_set(desstrinan) then dessinetri, trinan, xnan, ynan, color = colortrinan
            endif
         endif
;------------------------------------------------------------
; IV) trace les cotes en trait de couleur c_cote
;------------------------------------------------------------

         if NOT keyword_set(cont_thick) then cont_thick = 1
         IF chkstru(ex, 'C_THICK') THEN ex.C_THICK = cont_thick
;
         if n_elements(key_stride) LE 2 then key_stride = [1, 1, 1]
         key_stride = 1l > long(key_stride)
;
         if (keyword_set(forplt) AND (!map.projection GT 0 OR key_irregular)) $
          OR keyword_set(nan) then $
          tracecote, CONT_THICK = cont_thick, _extra = ex $
         ELSE tracemask, mask, xm, ym, CONT_THICK = cont_thick, _extra = ex
;
      endif
   ENDIF
;------------------------------------------------------------
fini:
   if keyword_set(desstrichamp) then dessinetri, trichamp, x, y, color = colortrichamp, thick = th_trichamp
   if keyword_set(desstrimask) then dessinetri, trimsk, xm, ym, color = colortrimask, thick = th_trimask
   IF keyword_set(key_performance) THEN print, 'temps pltbase', systime(1)-tempsun 

   return
end
